﻿using Infragistics.Controls.Schedules;
using Infragistics;
using KB9601_XamGantt_SettingHolidayAndWorkingDay.Infrastructure;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace KB9601_XamGantt_SettingHolidayAndWorkingDay
{
    internal class MainWindowViewModel : ObservableObject
    {
        private Project? _project;

        public Project Project
        {
            get
            {
                if (_project == null)
                {
                    _project = GenerateExceptionCalendarProject();
                }
                return _project;
            }
            set { _project = value; OnPropertyChanged(); }
        }

        public MainWindowViewModel()
        {
        }

        public Project GenerateExceptionCalendarProject()
        {
            Project project = new Project();
            ProjectCalendar projectCalendar = new ProjectCalendar() { UniqueId = "pce" };

            // プロジェクト開始日
            project.Start = new DateTime(2020, 12, 1);

            // https://jp.infragistics.com/help/wpf/xamgantt-code-example-create-calendar-exceptions

            // ==== 毎年5月3日をプロジェクト開始日から2年間非稼働日にする場合 ====
            ProjectCalendarException pce1 = new ProjectCalendarException();
            pce1.DateRange = new DateRange(project.Start, project.Start.AddYears(2)); // 開始日: プロジェクト開始日、終了日: プロジェクト開始日の2年後
            pce1.DaySettings = new DaySettings { IsWorkday = false };   // 非稼働日として設定
            pce1.Recurrence = new DateRecurrence    // 繰り返しの設定
            {
                Count = 0,  // 無期限（※無期限の場合、DateRangeは開始日と終了日の両方の指定が必要）
                Frequency = DateRecurrenceFrequency.Yearly  // 年次
            };
            pce1.Recurrence.Rules.Add(new MonthOfYearRecurrenceRule(5));    // 毎年5月
            pce1.Recurrence.Rules.Add(new DayOfMonthRecurrenceRule(3)); // 毎月3日（※MonthOfYearRecurrenceRuleとDayOfMonthRecurrenceRuleの両方同時にDateRecurrenceFrequencyをYearlyで追加することで、毎年5月3日を非稼働日に指定できる）
            projectCalendar.Exceptions.Add(pce1);   // プロジェクトのカレンダーに追加する。

            // ==== 2021年1月30日(土)のみを稼働日にする場合（特定の日のみを非稼働日にする場合は、IsWorkdayをfalseにする以外は、同様です） ====
            ProjectCalendarException pce2 = new ProjectCalendarException();
            pce2.DateRange = new DateRange(new DateTime(2021, 1, 30)); // 開始日を指定
            pce2.DaySettings = new DaySettings { IsWorkday = true };   // 稼働日として設定
            pce2.Recurrence = new DateRecurrence    // 繰り返しの設定
            {
                Count = 1,  // 1回だけ
                Frequency = DateRecurrenceFrequency.Daily   // 日次
            };
            projectCalendar.Exceptions.Add(pce2);   // プロジェクトのカレンダーに追加する。

            project.Calendars.Add(projectCalendar);
            project.CalendarId = "pce";
            
            AddSampleWorkingHours(project);
            return project;
        }

        // これはヘルパー メソッド。10個のタスクを追加し関係を構築する
        private Project AddSampleWorkingHours(Project project)
        {
            ProjectTask root = new ProjectTask
            {
                TaskName = "Summary",
                IsManual = false
            };
            project.RootTask.Tasks.Add(root);
            for (int i = 0; i < 10; i++)
            {
                root.Tasks.Add(new ProjectTask
                {
                    TaskName = String.Format("Task {0:00}", i.ToString()),
                    IsManual = false,
                    ManualDuration = ProjectDuration.FromFormatUnits(8.0, ProjectDurationFormat.Hours),
                    Notes = String.Format("Notes {0:00}", i.ToString())
                });
            }
            for (int i = 1; i < 10; i++)
            {
                root.Tasks[i].Predecessors.Add(root.Tasks[i - 1]);
            }
            return project;
        }
    }
}
